#include <stdio.h>
#include <stdint.h>
#include <error.h>

#define LENGTH 29
#define BYTES ((LENGTH + 7) / 8)

uint8_t buffer[BYTES] = { 0, 0, 0, 0 };

void check_index (int i)
{
  if (i < 0 || i >= LENGTH)
    error (1, 0, "index %d out of range (0, ..., %d)", i, LENGTH - 1);
}

void bit_array_set (int i, int value)
{
  check_index (i);
  int byte_index = i / 8;
  int bit_index = i % 8;
  uint8_t mask = 1 << bit_index;
  if (value)
    buffer[byte_index] |= mask;
  else
    buffer[byte_index] &= ~mask;
}

void bit_array_flip (int i)
{
  check_index (i);
  int byte_index = i / 8;
  int bit_index = i % 8;
  uint8_t mask = 1 << bit_index;
  buffer[byte_index] ^= mask;
}

int bit_array_get (int i)
{
  check_index (i);
  int byte_index = i / 8;
  int bit_index = i % 8;
  uint8_t mask = 1 << bit_index;
  if (buffer[byte_index] & mask)
    return 1;
  else
    return 0;
}

void output (void)
{
  for (int i = 0; i < LENGTH; i++)
    {
      if (i % 4 == 0)
        printf (" ");
      printf ("%d", bit_array_get (i));
    }
  printf ("\n");
}

int main (void)
{
  output ();
  bit_array_set (12, 1);
  output ();
  bit_array_flip (28);
  output ();
  bit_array_set (12, 0);
  output ();
  bit_array_flip (31);
  output ();
}
