#include <stdio.h>
#include <stdint.h>
#include <error.h>
#include <stdlib.h>

int length = 0;
int bytes = 0;
uint8_t *buffer = NULL;

void bit_array_init (int n)
{
  length = n;
  bytes = (length + 7) / 8;
  if (buffer)
    free (buffer);
  buffer = malloc (bytes * sizeof (uint8_t));
  for (int i = 0; i < bytes; i++)
    buffer[i] = 0;
}

void bit_array_resize (int new_n)
{
  length = new_n;
  bytes = (length + 7) / 8;
  buffer = realloc (buffer, bytes * sizeof (uint8_t));
}

void bit_array_done (void)
{
  free (buffer);
  buffer = NULL;
  length = 0;
  bytes = 0;
}

void check_index (int i)
{
  if (!buffer)
    error (1, 0, "array not initialised");
  if (i < 0 || i >= length)
    error (1, 0, "index %d out of range (0, ..., %d)", i, length - 1);
}

void bit_array_set (int i, int value)
{
  check_index (i);
  int byte_index = i / 8;
  int bit_index = i % 8;
  uint8_t mask = 1 << bit_index;
  if (value)
    buffer[byte_index] |= mask;
  else
    buffer[byte_index] &= ~mask;
}

void bit_array_flip (int i)
{
  check_index (i);
  int byte_index = i / 8;
  int bit_index = i % 8;
  uint8_t mask = 1 << bit_index;
  buffer[byte_index] ^= mask;
}

int bit_array_get (int i)
{
  check_index (i);
  int byte_index = i / 8;
  int bit_index = i % 8;
  uint8_t mask = 1 << bit_index;
  if (buffer[byte_index] & mask)
    return 1;
  else
    return 0;
}

void output (void)
{
  for (int i = 0; i < length; i++)
    {
      if (i % 4 == 0)
        printf (" ");
      printf ("%d", bit_array_get (i));
    }
  printf ("\n");
}

int main (void)
{
  bit_array_init (29);
  output ();
  bit_array_set (12, 1);
  output ();
  bit_array_flip (28);
  output ();
  bit_array_set (12, 0);
  output ();
  bit_array_flip (31);
  output ();
  bit_array_done ();
}
