#include <stdio.h>
#include <stdlib.h>

typedef struct node
{
  int content;
  struct node *next, *prev;
} node;

void check_list (node *first)
{
  for (node *p = first; p; p = p->next)
    {
      if (p->next && p->next->prev != p)
        fprintf (stderr, "List inconsistency!\n");
      if (p->prev && p->prev->next != p)
        fprintf (stderr, "List inconsistency!\n");
    }
}

void output_list (node *first)
{
  for (node *p = first; p; p = p->next)
    printf ("%d ", p->content);
  printf ("\n");
}

void insert_into_list (node *what, node *where)
{
  what->next = where->next;
  if (where->next)
    where->next->prev = what;
  what->prev = where;
  where->next = what;
}

void delete_from_list (node *what, node **first)
{
  if (what == *first)
    {
      *first = what->next;
      if (*first)
        (*first)->prev = NULL;
    }
  else
    {
      node *p = *first;
      while (p && p->next != what)
        p = p->next;
      if (p)
        p->next = what->next;
      if (what->next)
        what->next->prev = p;
    }
  free (what);
}

int main (void)
{
  node *element3 = malloc (sizeof (node));
  node *element7 = malloc (sizeof (node));
  node *element137 = malloc (sizeof (node));

  element3->content = 3;
  element7->content = 7;
  element137->content = 137;

  node *first = element3;
  element3->prev = NULL;
  element3->next = element7;
  element7->prev = element3;
  element7->next = element137;
  element137->prev = element7;
  element137->next = NULL;

  output_list (first);
  check_list (first);

  node *element5 = malloc (sizeof (node));
  element5->content = 5;
  insert_into_list (element5, element3);

  output_list (first);
  check_list (first);

  delete_from_list (element5, &first);
  output_list (first);
  check_list (first);
  delete_from_list (element3, &first);
  output_list (first);
  check_list (first);
  delete_from_list (element137, &first);
  output_list (first);
  check_list (first);

  return 0;
}
